import {
  select,
  csv,
  json,
  scaleLinear,
  extent,
  scaleTime,
  timeFormat,
  scaleOrdinal,
  axisLeft,
  axisBottom,
  schemeCategory10,
  min,
  max,
  format,
} from 'd3';

const width = 900;
const height = 300;
const padding = 60;

const margin = { top: 50, right: 20, bottom: 35, left: 140 };
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;

const colorScale = d3
  .scaleOrdinal()
  .domain([
    '14',
    '13',
    '12',
    '11',
    '10',
    '9',
    '8',
    '7',
    '6',
    '5',
    '4',
    '3',
    '2',
    '1',
  ])
  .range([
    '#5e1f1f',
    '#993333',
    '#cc0000',
    '#ff5050',
    '#ff6666',
    '#ff9999',
    '#ffcccc',
    '#ffffff',
    '#ccffff',
    '#66ffff',
    '#00ffff',
    '#33cccc',
    '#006699',
    '#003366',
  ]);

const colors = [
  '#5e1f1f',
  '#993333',
  '#cc0000',
  '#ff5050',
  '#ff6666',
  '#ff9999',
  '#ffcccc',
  '#ffffff',
  '#ccffff',
  '#66ffff',
  '#00ffff',
  '#33cccc',
  '#006699',
  '#003366',
].reverse();

const months = [
  'January',
  'February',
  'March',
  'April',
  'May',
  'June',
  'July',
  'August',
  'September',
  'October',
  'November',
  'December',
];

const url =
  'https://gist.githubusercontent.com/mandadi1999/797ebe3b6f93b9e7da4062d64ae45457/raw/bee9a547dfa4a35cc918fa17ce1ca3446cc73450/heatmap.json';

const fetchAndRender = async () => {
  const response = await fetch(url);
  const text = await response.json();
  const { baseTemperature, monthlyVariance } = text;

  const dataset = monthlyVariance.map((d) => ({
    ...d,
    temp:
      d.variance > 0
        ? (baseTemperature + d.variance).toFixed(2)
        : (baseTemperature - Math.abs(d.variance)).toFixed(2),
  }));

  createViz(dataset);
};

fetchAndRender();

function createViz(data) {
  const cellHeight = (height - 2 * padding) / 12;
  const cellWidth = width / Math.floor(data.length / 13);

  const yScale = scaleTime()
    .domain([0, 11])
    .range([padding, height - padding]);

  const xScale = scaleTime()
    .domain([min(data, (d) => d.year), max(data, (d) => d.year)])
    .range([padding, width - padding]);

  const tempScale = scaleLinear().domain([1, 14]).range([0, 14]);

  const svg = d3
    .select('body')
    .append('svg')
    .attr('viewBox', `0 0 ${width} ${height}`);

  const toolTipContent = function (d) {
    document.getElementById('tooltip').setAttribute('data-year', d.year);
    const monthYear = timeFormat('%B %Y')(new Date(d.year, d.month - 1, 1));
    return (
      monthYear +
      '<br/>' +
      'Temperature ' +
      d.temp +
      '°C' +
      '<br/>' +
      'Variance ' +
      d.variance
    );
  };

  var tip = d3
    .tip()
    .attr('class', 'd3-tip')
    .attr('id', 'tooltip')
    .html((d) => toolTipContent(d));

  svg.call(tip);

  svg
    .selectAll('rect')
    .data(data)
    .enter()
    .append('rect')
    .attr('class', 'cell')
    .attr('data-month', (d) => d.month - 1)
    .attr('data-year', (d) => d.year)
    .attr('data-temp', (d) => d.temp)
    .attr('fill', (d) => colors[Math.floor(tempScale(d.temp))])
    .attr('x', (d) => xScale(d.year))
    .attr('y', (d) => yScale(d.month - 1) - cellHeight)
    .attr('width', cellWidth)
    .attr('height', cellHeight)
    .on('mouseover', tip.show)
    .on('mouseout', tip.hide);

  // create axis
  const xAxis = axisBottom(xScale).tickFormat(format('d'));
  const yAxis = axisLeft(yScale).tickFormat((month) => {
    const date = new Date('August 19, 1975 23:15:30');
    date.setMonth(month);
    return timeFormat('%B')(date);
  });

  svg
    .append('g')
    .attr('id', 'x-axis')
    .attr('transform', `translate(0, ${height - padding})`)
    .call(xAxis);

  svg
    .append('g')
    .attr('id', 'y-axis')
    .attr('transform', `translate(${padding}, ${-cellHeight})`)
    .call(yAxis);

  const colorLegend = (selection, props) => {
    const { colorScale, spacing, textOffset } = props;

    const groups = selection.selectAll('g').data(colorScale.domain().reverse());

    const groupsEnter = groups
      .enter()
      .append('g')
      .attr('class', 'tick')
      .attr('id', 'legend');

    groupsEnter
      .merge(groups)
      .attr('transform', (d, i) => `translate(${i * spacing}, 0)`);

    groups.exit().remove();

    // nested version of general update pattern
    groupsEnter
      .append('rect')
      .attr('id', 'legend')
      .merge(groups.select('rect'))
      .attr('x', -22)
      .attr('y', -15)
      .attr('width', 40)
      .attr('height', 30)
      .attr('fill', colorScale);

    groupsEnter
      .append('text')
      .merge(groups.select('text'))
      .text((d) => d)
      .attr('dy', '0.32em')
      .attr('fill', '#8e88883')
      .attr('stroke', 'black')
      .attr('x', textOffset);
  };

  svg
    .append('g')
    .attr('transform', `translate(${innerWidth / 4},${innerHeight + padding})`)
    .call(colorLegend, {
      colorScale,
      spacing: 40,
      textOffset: -8,
    });

  //   // create the legend

}